<?php

 
class Authenticate {

	private $redirectionURL;
	
	// When you create an object (using this class), it will force a redirection (if the user isn't logged in).
	// If you aren't sure if the user is logged in (and you want to passively check), then call the static method.
	function __construct(){

		$this->redirectionURL = "";

		$this->ensureLoggedIn();
	}

	// Returns True or false depending on whether the User is Logged in or not.
	// It only checks for the Account ID within the user's session variable.
	// This method will not tell you if the user "can be" logged in (based upon their permanent cookies).
	static function checkIfLoggedIn(){
		
		$accountID = WebUtil::GetSessionVar('AccountIdLoggedIn', "");
		
		$accountID = intval($accountID);
		
		if($accountID > 0){
			return true;
		}
		
		return false;
	}
	
	// If the AccountName/password is valid, this method will return the string "OK".
	// If the string does not say OK, the string will be some form of an error message.
	static function checkUserNamePass($inventorName, $password){
		
		$inventorObj = new Inventor();
		if(!$inventorObj->checkIfNameExists($inventorName)){
			return "Your user name was not found.";
		}
		
		$inventorID = $inventorObj->getInventorID($inventorName);
		
		if($inventorObj->checkInventorCredentials($inventorID, $password)){
			return "OK";
		}
		else{
			return "Your password is invalid.";
		}
	}
	
	// Call this function to Log someone In by their AccountID.
	// Pass in the $permanentlyFlag as TRUE to remember people on future visits with a permanent cookie.
	static function setAccountIDLoggedIn($accountID, $permanentFlag = false){
		
		if(!preg_match('/^\d+$/',$accountID)){
			throw new Exception("The Account ID must be numeric");
		}
		
		$accountID = intval($accountID);
		
		$inventorObj = new Inventor();
		$inventorObj->loadInventor($accountID);

		WebUtil::SetSessionVar('AccountIdLoggedIn', $accountID);
		
		if($permanentFlag){
			WebUtil::setCookie("PreAuthaccountID", $accountID, 360);
			WebUtil::setCookie("PreAuthUserPW", md5($inventorObj->getPassword() . Constants::getSecuritySalt()), 360);
		}
	}

	static function logOut(){
		
		$cookieTime = time() - 360;
		setcookie ("PreAuthaccountID", "", $cookieTime);
		setcookie ("PreAuthUserPW", "", $cookieTime);
		
		WebUtil::SetSessionVar("AccountIdLoggedIn", "");
	}


	// Will return the Account ID that is logged in.
	// Ensure they are logged in before calling this method!
	function getAccountID(){

		$accountID = WebUtil::GetSessionVar('AccountIdLoggedIn', "");
		
		if (!preg_match('/^\d{1,11}$/', $accountID) || empty($accountID)){
			throw new Exception("User must be logged in before calling the method getAccountID");
		}

		return $accountID;
	}
	
	// This is the place that people would be going after logging in.
	// By default, take them to the place they were trying to get when they were first sent to the Login Page.
	function getDefaultRedirectionURL(){
		
		$redirectURL = basename($_SERVER['PHP_SELF']) . "?" . $_SERVER['QUERY_STRING'];
		
		return $redirectURL;
	}
	
	function redirectUserToSignInPage(){
		
		if(empty($this->redirectionURL))
			$signInRedirectAddress = $this->getDefaultRedirectionURL();
		else
			$signInRedirectAddress = $this->redirectionURL;
		
		$redirectURL = "/login.php?forward=" . urlencode($signInRedirectAddress);
			
		// Close Session lock as soon as possible.
		// This is more important when session management is moved to the DB.
		session_write_close();
		
		header("Location: " . $redirectURL);
		
		exit;
	}
	
	// This will return TRUE if the ID/PW in the cookie is valid and it will set a Session Cookie with the account ID.
	// If this method returns FALSE, the account holder must type in their password again.
	static function autoLoginFromCookie(){
		
			$preAuthaccountID = WebUtil::GetCookie("PreAuthaccountID");
			$preAuthUserPW = WebUtil::GetCookie("PreAuthUserPW");
			
			$preAuthaccountID = intval($preAuthaccountID);

			if(empty($preAuthaccountID)){
				return false;
			}

			$inventorObj = new Inventor();
			
			if(!$inventorObj->checkIfExists($preAuthaccountID)){
				return false;
			}
			
			$inventorObj->loadInventor($preAuthaccountID);
			
			// If the cookie password matches, we will set their Account ID within the session variable.
			if(md5($inventorObj->getPassword() . Constants::getSecuritySalt()) == $preAuthUserPW){		
				
				self::setAccountIDLoggedIn($preAuthaccountID);
				
				return true;
			}
			
			return false;
	}

	// Looks for an Account ID in the session variable first.
	// If the user is not logged in, this method will attempt to find their credentials within a permanent cookie.
	private function verifyCredentials(){

		// Contains the session variable for the Account ID of the person who is logged in.
		$accountIdLoggedIn = WebUtil::GetSessionVar('AccountIdLoggedIn', "");

		// A session variable named "AccountIdLoggedIn" will always tell us if the account holder is logged in (it will have their Account ID).
		// If we are remembering someone's password for return visits, 
		// there will be a permanent cookie on their machine containing their Account ID, as well as an MD5 hash of their password.
		if (!empty($accountIdLoggedIn)) {
			
			// A fairly unnecessary level of protection.
			if (!preg_match('/^\d{1,11}$/', $accountIdLoggedIn)){
				return false;
			}
			
			// This means that they are logged in with a session variable.
			return true;
		}
		if (empty($accountIdLoggedIn)) {
		
			if(self::autoLoginFromCookie()){
				return true;
			}
		}
		
		// Security principals require us to default to the most restrictive "return value".
		// A coding mishap (like mixing up curly brackets) could jeopardize the security.
		return false;
	}

	private function ensureLoggedIn(){
		if(!$this->verifyCredentials()){
			$this->redirectUserToSignInPage();
		}
	}
}



