<?php



// This is Stuff that I could have put in DbCmd.
// I moved the less common methods into this class to reduce the memory footprint. 
// DbCmd can have thousands of objects of itself created in a very short time.
class DbHelper {

    // A private constructor; prevents direct creation of object
    private function __construct() { }

	// Method escapes value depending on "data type".
	// Method automatically converts date/time to DB formats.
	// To reduce marshaling, parameters are passed (and returned) by reference.
	static function &quoteValue($type, &$value) {
		if ($value === NULL || $value === "") {
			$returnValue = "NULL";
			return $returnValue;
		}
		
		$returnValue = null;
		
		switch ($type) {
			
			// For dates... try to determine if this is in Mysql Date format or a UnixTimestamp
			// An integer 14 digits long is probably in mysql format like ... '20060628024527'
			// A unix timestamp will never go over 11 digits long (after 2040 or whenever) like in 2006 the current time is 1151487927

			case "datetime" :
			case "timestamp" :
				
				if (preg_match ( '/^\d{14}$/', $value )){
					$returnValue = "'" . $value . "'"; 
				}
				else if (preg_match ( '/^\d{10,11}$/', $value )){
					$returnValue = "'" . date ( "YmdHis", $value ) . "'"; 
				}
				else {
					// Convert from Unix time stamp to Mysql format.
					if (preg_match ( '/^\d{4}\-\d{2}\-\d{2}\s\d{2}:\d{2}:\d{2}$/', $value )){
						$returnValue = "'" . $value . "'"; 
					}
					else{
						throw new Exception("Error inserting DateTime or TimeStamp data type into the Database. It is not a valid date." );
					}
				}
			
				break;
			
			case "date" :
				
				if (preg_match ( '/^\d{14}$/', $value ) || preg_match ( '/^\d{8}$/', $value )){
					$returnValue = "'" . $value . "'"; 
				}
				else if (preg_match ( '/^\d{10,11}$/', $value )){
					$returnValue = "'" . date ( "Ymd", $value ) . "'"; 
				}
				else {
					// Convert from Unix time stamp to Mysql format.
					if (preg_match ( '/^\d{4}\-\d{2}\-\d{2}$/', $value )){
						$returnValue = "'" . $value . "'"; 
					}
					else{
						throw new Exception("Error inserting Date data type into the Database. It is not a valid date: $value" );
					}
				}
				
				break;
				
			case "time" :
				
				if (preg_match ( '/^\d{6}$/', $value )){
					$returnValue = "'" . $value . "'"; 
				}
				else if (preg_match ( '/^\d{10,11}$/', $value )){
					$returnValue = "'" . date ( "His", $value ) . "'"; 
				}
				else{
					// Convert from Unix time stamp to Mysql format.
					if (preg_match ( '/^\d{2}:\d{2}:\d{2}$/', $value )){
							$returnValue = "'" . $value . "'"; 
					}
					else{
						throw new Exception("Error inserting Time data type into the Database. It is not a valid date." );
					}
				}
								
				break;
			
			case "string" :
			case "blob" :
			case "tinyblob" :
			case "mediumblob" :
			case "longblob" :	
			case "text" :
			case "tinytext" :	
			case "mediumtext" :
			case "longtext" :
			case "char" :
			case "varchar" :
				
				$returnValue = "'" . mysql_real_escape_string ( $value ) . "'";
				break;
				
			case "int" :
			case "integer" :
			case "mediumint" :
			case "bigint" :
			case "unsigned" :
				
				if (! preg_match ( '/^\-?\d{1,50}$/', $value )){
					throw new Exception("Error inserting Numeric data type into the Database. It is not a valid integer." );
				}
				
				$returnValue = $value;
				break;
				
			case "tinyint" :
				
				$returnValue = "'" . ((!empty( $value ) || $value === true) ? "1" : "0"). "'";
				
				if(strtoupper($value) === "FALSE" || $value === "0"){
					$returnValue = "'0'";
				}
				
				break;
				
			case "real" :
			case "double" :
			case "float" :
				
				if (! preg_match ( '/^\-?\d{1,50}(\.\d{1,50})?$/', $value )){
					throw new Exception("Error inserting numeric data type into the Database. It is not a valid decimal number." );
				}
				
				$returnValue = $value;
				break;
				
			default :
				throw new Exception("Undefined Type in Database Driver: $type : $value" );
		
		}
		
		return $returnValue;
	}
	
	// Fetch "field types" out of the table.
	// Returns an array with keys as field names, values as field types
	// Types are "string" "int", "timstampe", etc.
	static function &getFieldTypes(&$dbConnection, $table) {
		
		$field_names = array ();
		$columnsquery = "SHOW COLUMNS FROM " . mysql_escape_string ( $table );
		
		$res = mysql_query ( $columnsquery, $dbConnection );
		if ($res == FALSE){
			throw new Exception("Error in method getFieldTypes, the Table Name does not exist." );
		}
		
		$fieldsArr = array();
		while ( $fieldsArr = mysql_fetch_row ( $res ) ) {
			
			// Strip off any sizes... such as "int(11)" to just "int"
			$colTypeParts = explode ( "(", $fieldsArr [1] );
			$columnType = rtrim ( $colTypeParts [0], ")" );
			
			// The key is the column name... the value is the column type.
			$field_names [$fieldsArr [0]] = $columnType;
		}

		return $field_names;
	}
	
	
	// This method will build a string for the "update query" and return it by reference.
	static function &getUpdateQueryStr($dbConnection, $tableName, array &$fields, $qualifier) {
		
		$fldTerms = "";
		$fieldInfo =& self::getFieldTypes ($dbConnection, $tableName );
		
		for(reset ( $fields ); $name = key ( $fields ); next ( $fields )) {
			
			if (! isset ( $fieldInfo [$name] )){
				throw new Exception("Unknown Column Name for Update Query: $name" );
			}
			
			$escapedValue =& self::quoteValue ($fieldInfo[$name], $fields[$name]);
			$fldTerms .= $name . "=";
			$fldTerms .= $escapedValue . ",";
		}
		
		$fldTerms = rtrim ( $fldTerms, "," ); //Remove last comma
		
		$queryString = "UPDATE $tableName SET $fldTerms  WHERE $qualifier";
		
		return $queryString;
	}
	
	
	// This method will build a string for the "insert query" and return it by reference.
	static function &getInsertQueryStr($dbConnection, $tableName, array &$fields) {
		
		$fldNames = "";
		$fldValues = "";
		$fieldInfo =& self::getFieldTypes($dbConnection, $tableName );
		
		for(reset ( $fields ); $name = key ( $fields ); next ( $fields )) {
			
			if (! isset ( $fieldInfo [$name] )){
				throw new Exception("Unknown Column Name for Insert Query: $name" );
			}
			
			$escapedValue =& self::quoteValue ( $fieldInfo [$name], $fields [$name] );
			
			$fldNames .= $name . ",";
			$fldValues .= $escapedValue . ",";
		}
		
		$fldNames = rtrim ( $fldNames, "," ); //Remove last comma
		$fldValues = rtrim ( $fldValues, "," ); //Remove last comma
		
		$queryString = "INSERT INTO $tableName ($fldNames) VALUES ($fldValues)";
		
		return $queryString;
	}
	
	// Pass in something like DBhelper::getOrClauseFromList("users.DomainID", array(1,2,3))
	// This method will return "(users.DomainID='1' OR users.DomainID='2' OR users.DomainID='3')
	// Returns parentheses if the array is not empty.  Otherwise returns a blank string.
	static function getOrValuesForColumn($columnName, array $valuesArr){
		
		$retStr = "";
		
		foreach($valuesArr as $thisValue){
			
			if(!empty($retStr)){
				$retStr .= " OR ";
			}
				
			$retStr .= DbCmd::escapeSQL($columnName) . '="'.DbCmd::escapeSQL($thisValue).'"';
		}
		
		if(!empty($retStr)){
			$retStr = " (" . $retStr . ") ";
		}
		
		return $retStr;
	}
	// Pass in something like DBhelper::getOrColumnsFromArray(array('Col1', 'Col2', 'Col3'), 'value')
	// This method will return "(Col1='value' OR Col2='value' OR Col3='value')
	// Returns parentheses if the array is not empty.  Otherwise returns a blank string.
	static function getOrColumnsForValue(array $columnNames, $value){
		
		$retStr = "";
		
		foreach($columnNames as $thisColumn){
			
			if(!empty($retStr)){
				$retStr .= " OR ";
			}
				
			$retStr .= DbCmd::escapeSQL($thisColumn) . '="'.DbCmd::escapeSQL($value).'"';
		}
		
		if(!empty($retStr)){
			$retStr = " (" . $retStr . ") ";
		}
		
		return $retStr;
	}
}




