<?php



include_once 'Classes/Triangle.php';

class Structure {

	private $dbCmd;
	private $tableStructure;
	private $characterCodeCount;
	private static $singleton;
	
	// Make variable static to prevent multiple database lookups during a single thread.
	private static $characterCodesArr = array();
	
	// Define SQL to create a table representing a pyramidal level.
	// {CHARACTER_CODE} is a variable to be searched/replaced within the SQL.
	function __construct(){
		
		$this->dbCmd = new DbCmd();
		
		$this->tableStructure = "
			CREATE TABLE IF NOT EXISTS t{CHARACTER_CODE} (
				ID int(11) NOT NULL AUTO_INCREMENT,
				
				# ID serves the same purpose for sorting.
				# The Date column will allow the application to work with replicated data.
				Date DATETIME,
				
				ChildSignature char(32),
				Signature char(32),
				
				L6		char(32),
				L5		char(32),
				L4		char(32),
				L3		char(32),
				L5_G01	char(32),
				L4_G01	char(32),
				L3_G01	char(32),
				L4_G02	char(32),
				L3_G02	char(32),
				L3_G03	char(32),
				L2		char(32),
				L5_G1	char(32),
				L4_G1	char(32),
				L4_G101	char(32),
				L3_G1	char(32),
				L4_G11	char(32),
				L3_G11	char(32),
				L3_G12	char(32),
				L2_G1	char(32),
				L4_G2	char(32),
				L3_G2	char(32),
				L3_G21	char(32),
				L2_G2	char(32),
				L3_G3	char(32),
				L2_G3	char(32),
				L2_G4	char(32),
				L1		char(32),
				
				PRIMARY KEY (ID),
				
				INDEX i{CHARACTER_CODE}_ChildSignature (ChildSignature),
				INDEX i{CHARACTER_CODE}_Signature (Signature),
				
				INDEX i{CHARACTER_CODE}_L6 (L6),
				INDEX i{CHARACTER_CODE}_L5 (L5),
				INDEX i{CHARACTER_CODE}_L4 (L4),
				INDEX i{CHARACTER_CODE}_L3 (L3),
				INDEX i{CHARACTER_CODE}_L5_G01 (L5_G01),
				INDEX i{CHARACTER_CODE}_L4_G01 (L4_G01),
				INDEX i{CHARACTER_CODE}_L3_G01 (L3_G01),
				INDEX i{CHARACTER_CODE}_L4_G02 (L4_G02),
				INDEX i{CHARACTER_CODE}_L3_G02 (L3_G02),
				INDEX i{CHARACTER_CODE}_L3_G03 (L3_G03),
				INDEX i{CHARACTER_CODE}_L2 (L2),
				INDEX i{CHARACTER_CODE}_L5_G1 (L5_G1),
				INDEX i{CHARACTER_CODE}_L4_G1 (L4_G1),
				INDEX i{CHARACTER_CODE}_L4_G101 (L4_G101),
				INDEX i{CHARACTER_CODE}_L3_G1 (L3_G1),
				INDEX i{CHARACTER_CODE}_L4_G11 (L4_G11),
				INDEX i{CHARACTER_CODE}_L3_G11 (L3_G11),
				INDEX i{CHARACTER_CODE}_L3_G12 (L3_G12),
				INDEX i{CHARACTER_CODE}_L2_G1 (L2_G1),
				INDEX i{CHARACTER_CODE}_L4_G2 (L4_G2),
				INDEX i{CHARACTER_CODE}_L3_G2 (L3_G2),
				INDEX i{CHARACTER_CODE}_L3_G21 (L3_G21),
				INDEX i{CHARACTER_CODE}_L2_G2 (L2_G2),
				INDEX i{CHARACTER_CODE}_L3_G3 (L3_G3),
				INDEX i{CHARACTER_CODE}_L2_G3 (L2_G3),
				INDEX i{CHARACTER_CODE}_L2_G4 (L2_G4),
				INDEX i{CHARACTER_CODE}_L1 (L1)
			) ENGINE=MYISAM;
			";
	}
	
	static function getSingleton(){
		
		if(empty(self::$singleton)){
			self::$singleton = new Structure();
		}
		
		return self::$singleton;
	}
	
	// Define acceptable control characters to build pyramids with.
	static function getControlCharacters(){
		return array(ord(" "), ord("\n"), ord("\t"));
	}
	
	// This will create the necessary tables to store data in a pyramidal structure.
	// The names of the tables are based upon the characters (ASCII Codes) where which the hierarchy diverges.
	// Characters/letters/words are never actually written to the database.
	// Content can be rebuilt by inferring characters from the table names.
	// Unicode may be substituted for ASCII, but the table count would be impractical. 
	// Limit the gammut of Unicode characters by running the cardinality script on a training file.
	// To make the system work with vision or audio, WAV forms & Pixels may be converted to ASCII representations.
	// For example convert each pixel to a 254 color pallete (mapped to specific ASCII codes). Reserve a character for "line breaks" within the pixel rows.
	// Vision may require multiple databases with different cardinality sets for specific duties (including low-light situations).
	// The application is not designed to handle multiple cardinality sets yet... but the algorithm won't have to change much.
	// Each cardinality set will become the building blocks of even larger pyramids which work together.  For example, vision, touch, and smell.
	function buildPyramidalStructure(){
		
		$this->loadCharacterCodes();
		
		// Each pyramid has columns which are used to identify its relationship to all of the pyramids which are on top.
		$parentColumns = array();
		$parentIndexes = array();
		
		foreach(self::$characterCodesArr as $loop_AsciiCode){
			
			$loop_TableSQL = $this->tableStructure;
			
			$loop_TableSQL = preg_replace('/\{CHARACTER_CODE\}/', $loop_AsciiCode, $loop_TableSQL);
			
			// Get rid of Tabs.
			$loop_TableSQL = preg_replace('/\t/', " ", $loop_TableSQL);
			
			$this->dbCmd->query($loop_TableSQL);
			
		}
	}
	
	function resetDatabase(){
		
		$this->dbCmd->query("DROP TABLE IF EXISTS cardinality");
		
		$this->dbCmd->query("CREATE TABLE IF NOT EXISTS cardinality (
								ID int(11) NOT NULL AUTO_INCREMENT,
								CharacterCode int(11) NOT NULL,
							
								PRIMARY KEY (ID),
								UNIQUE c_CharacterCode (CharacterCode)
								) ENGINE=MYISAM
							");
		

		// There could be high character codes if the database was trained on a subset of unicode.
		for($i=0; $i<30000; $i++){
			$this->dbCmd->query("DROP TABLE IF EXISTS t" . $i);
		}
	}
	
	// Pass in an array to train the database on.
	// The more popular the occurance is, the closer the code should be to the front of the array.
	function setCardinality(array $characterCodesArr){
		
		if(sizeof($characterCodesArr) < 3){
			throw new Exception("There must be at least 3 values within a cardinality set in order to detect a pattern.");
		}
		
		foreach($characterCodesArr as $loop_CharacterCode){
			
			if(!preg_match('/^\d+$/', $loop_CharacterCode)){
				throw  new Exception("The training set must be numeric.");
			}
			
			$this->dbCmd->insertQuery("cardinality", array("CharacterCode"=>$loop_CharacterCode));
		}
	}
	
	function checkIfCardinalityHasBeenSet(){
		
		$this->dbCmd->query("SELECT COUNT(*) FROM cardinality");
		if($this->dbCmd->getValue() > 0){
			return true;
		}
		else{
			return false;
		}
	}
	
	// This will return a string with characters that are known to be in our training set.
	// If there are characters outside of our gammut, the "unknown symbol" will be substituted.
	// If the unknown symbol is a nuisance, filter it before outputting.
	function getCleanData($data){
		
		// Make sure that the data we are adding does not have any unknown characters.
		$returnData = "";
		
		$charsArr = preg_split('//', $data, -1, PREG_SPLIT_NO_EMPTY);
		
		$this->loadCharacterCodes();
		
		foreach($charsArr as $loop_Char){
			
			if(!in_array(ord($loop_Char), self::$characterCodesArr)){
				
				// Don't substitute an "unknown symbol" if it is a control character.
				if(ord($loop_Char) < 32){
					continue;
				}
				
				$returnData .= SYMBOL_UNKNOWN;
			}
			else{
				$returnData .= $loop_Char;
			}
		}
		
		return $returnData;
	}
	
	function getTotalCharacterCount(){
		
		$this->loadCharacterCodes();
		
		return $this->characterCodeCount;
	}
	
	// The position is Zero-based.
	// The first position represents the character with the highest cardinality.  With text, this is typically the space character.
	function getCharacterCodeByPosition($position){
		
		$this->loadCharacterCodes();
		
		$position = intval($position);
		
		if($position < 0 || $position > ($this->characterCodeCount - 1)){
			throw new Exception("The position is out of range.");
		}
		
		return self::$characterCodesArr[$this->characterCodeCount - $position - 1];
	}
	
	// Loads the character codes once out of the database and stores the values within a static array.
	// The last element of the array has the highest cardinality. With text, this is typically the space character.
	// The special "Unknown Character" will appear first in the array, since it should appear least often within input data.
	private function loadCharacterCodes(){
		
		if(!empty(self::$characterCodesArr)){
			return;
		}
		
		$this->dbCmd->query("SELECT CharacterCode FROM cardinality ORDER BY ID DESC");
		self::$characterCodesArr = $this->dbCmd->getValueArr();
		
		array_unshift(self::$characterCodesArr, ord(SYMBOL_UNKNOWN));
		
		$this->characterCodeCount = sizeof(self::$characterCodesArr);
	}
	
	// Returns a counter to describe the position.
	// The lower the cardinality, the lower the number.
	// Number is 1-based.
	function getPositionInCardinalityChain($characterCode){
		
		$this->loadCharacterCodes();
		
		$counter = 1;
		foreach(self::$characterCodesArr as $loop_CharCode){
			
			if($loop_CharCode == $characterCode){
				return $counter;
			}
			
			$counter++;
		}
		throw new Exception("Character code not found.");
	}
	
	// Returns 1 of 3 values.
	function getRelationshipInCardinalityChain($characterCode){
		
		$this->loadCharacterCodes();
		
		$tripleCounter = 0;
		foreach(self::$characterCodesArr as $loop_CharCode){
			
			$tripleCounter++;
			
			if($tripleCounter > 3){
				$tripleCounter = 1;
			}
			
			if($loop_CharCode != $characterCode){
				continue;
			}
			
			if($tripleCounter == 1){
				return RELATIONSHIP_SELF;
			}
			else if($tripleCounter == 2){
				return RELATIONSHIP_PARENTS;
			}
			else if($tripleCounter == 3){
				return RELATIONSHIP_GRANDPARENTS;
			}
			
		}
		
		throw new Exception("Character code not found.");
	}
	
	// For debugging it is helpful to indent (based upon the cardinality position).
	static function getSpacesFromCharacterCode($characterCode){
		
		$triangleObj = self::getSingleton();
		
		$cardinalityCount = $triangleObj->getPositionInCardinalityChain($characterCode);
		
		$retStr = "";
		for($i=0; $i<$cardinalityCount; $i++){
			$retStr .= "&nbsp;&nbsp;&nbsp;";
		}
		
		return $retStr;
	}
	
	
	// With text, this is typically the space character.
	function getCharacterCodeWithHighestCardinality(){
		
		$this->loadCharacterCodes();
		
		return end(self::$characterCodesArr);
	}
	function getCharacterCodeWithLowestCardinality(){
		
		$this->loadCharacterCodes();
		
		return reset(self::$characterCodesArr);
	}
	
	// Based upon the parent character code it is possible to infer the child.
	// If the $parentCharacterCode code is at the bottom level, this method will return null.
	function getCharacterCodeWithNextHighestCardinality($parentCharacterCode){
		
		$this->loadCharacterCodes();
		
		$lastCharacterCode = false;
		
		foreach(self::$characterCodesArr as $loop_Code){
			
			// This means that this last character code in the loop matched our $parentCharacterCode
			// Therefore the character code in this loop should be the child.
			if($lastCharacterCode){
				return $loop_Code;
			}
			
			if($loop_Code == $parentCharacterCode){
				$lastCharacterCode = true;
			}
		}
		
		if(!$lastCharacterCode){
			throw new Exception("The parent character code was not found.");
		}
		
		return null;
	}
	
	// Ensures that the character is in our cardinality database.
	function validateCharacter($character){

		$this->loadCharacterCodes();
		
		if($character == SYMBOL_END){
			return;
		}
		
		$checkCharacterCode = ord($character);
		
		foreach(self::$characterCodesArr as $loop_Code){
			if($loop_Code == $checkCharacterCode){
				return;
			}
		}
		
		throw new Exception("The character is invalid.");
		
	}

}


