<?php

 
// ---------------  Custom Filter Constants ------------------------

// Can only contain Letters or numbers and NO spaces.
// Does not allow any thing but a-z A-Z or 0-9
define("FILTER_SANITIZE_WORD_OR_NUMBER_NO_SPACES", "FILTER_SANITIZE_WORD_OR_NUMBER_NO_SPACES");

// Returns 0 if the parameter is null or empty.  Does not allow negative
define("FILTER_SANITIZE_INT", "FILTER_SANITIZE_INT");

// FILTER_SANITIZE_NUMBER_FLOAT has some unexpected behaviour, dealing with fractions and scientific notation.
define("FILTER_SANITIZE_FLOAT", "FILTER_SANITIZE_FLOAT");

// Such has peoples names, or phone numbers, etc. When there shouldn't be tabs, new lines, etc.
// This shoudl also be used for Keyword Searches, like in a search engine.
define("FILTER_SANITIZE_STRING_ONE_LINE", "FILTER_SANITIZE_STRING_ONE_LINE");

// Sometimes MS Word injects their own character codes, such as angled quotes, or triple periods.
define("FILTER_SANITIZE_MS_WORD_CHARACTERS", "FILTER_SANITIZE_MS_WORD_CHARACTERS");

// Multi line text boxes.  Maybe Shipping Instructions, or a chat window. 
// Makes sure there are no bad characters in there (other than \n newline characters).
define("FILTER_SANITIZE_STRING_MULTI_LINE", "FILTER_SANITIZE_STRING_MULTI_LINE");

class WebUtil {
		

	
	static function filterURL($urlToFilter){
		$urlToFilter = filter_var($urlToFilter, FILTER_SANITIZE_URL);
		return $urlToFilter;	
	}

	static function initializeSession(){
		
		if(session_id() != "")
			return;
			
		$bool = session_start();
		
		if (!$bool){
			exit("There is a problem starting the session.");
		}
	}
	
	static function getSessionID(){
		return session_id();
	}
	
	// Get and Process Request Input in specified variable within either $_GET or $_POST
	// Return nullValue if variable not present, else returns processed value
	static function getInput( $vname, $filterType, $nullValue = NULL ){
		
		// If the variable does not exist, return the default value.
		if( !array_key_exists( $vname, $_REQUEST )){
			return self::getDefaultNullValueBaseOnFilterType($filterType, $nullValue);
		}

		// Get value out of HTTP request
		$returnValue = trim($_REQUEST[$vname]);

		return WebUtil::filterData($returnValue, $filterType);
	}
	
	private static function getDefaultNullValueBaseOnFilterType($filterType, $nullValue){
		
		// Make sure we always return a number if we are asking for an INT. Even the default value.
		if($filterType == FILTER_SANITIZE_INT && !preg_match('/^\d{1,12}$/', $nullValue)){
			return 0;
		}
		else{
			return $nullValue;
		}
	}
	
	static function filterData($dataToFilter, $filterType){
		
		if($filterType == FILTER_SANITIZE_STRING_ONE_LINE){
			$dataToFilter = filter_var($dataToFilter, FILTER_SANITIZE_STRING, FILTER_FLAG_STRIP_LOW | FILTER_FLAG_NO_ENCODE_QUOTES);
		}
		else if($filterType == FILTER_SANITIZE_STRING_MULTI_LINE){
			
			// Split the multi line string into individual lines
			// On each line within a loop, filter it has if it was 1 line of data.
			// Then glue the "safe stuff" back together again.
			$dataLinesArr = preg_split("/\n/", $dataToFilter);
			
			$returnData = "";
			foreach($dataLinesArr as $thisLine){
				
				if(!empty($returnData)){
					$returnData .= "\n";
				}
				
				$returnData .= filter_var($thisLine, FILTER_SANITIZE_STRING, FILTER_FLAG_STRIP_LOW | FILTER_FLAG_NO_ENCODE_QUOTES);
			}
			
			$dataToFilter = $returnData;
		}
		else if($filterType == FILTER_SANITIZE_INT){
			
			$dataToFilter = intval($dataToFilter);
			
			if($dataToFilter < 0){
				$dataToFilter = 0;
			}
		}
		else if($filterType == FILTER_SANITIZE_EMAIL){
			$dataToFilter = filter_var($dataToFilter, FILTER_SANITIZE_EMAIL);
		}
		else if($filterType == FILTER_SANITIZE_STRING){
			$dataToFilter = filter_var($dataToFilter, FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
		}
		else if($filterType == FILTER_SANITIZE_NUMBER_INT){
			$dataToFilter = filter_var($dataToFilter, FILTER_SANITIZE_NUMBER_INT);
		}
		else if($filterType == FILTER_SANITIZE_FLOAT){
			
			$matches = array();
			
			if(preg_match('/(-?\d+(\.\d*)?)/', $dataToFilter, $matches)){
				$dataToFilter = $matches[1];
			}
			else{
				$dataToFilter = 0;
			}
				
			$dataToFilter = $dataToFilter + 0;
		}
		else if($filterType == FILTER_UNSAFE_RAW){
			// nothing
		}
		else if($filterType == FILTER_SANITIZE_MS_WORD_CHARACTERS){
			
			// Removes some bad characters that MS Word sometimes automatically converts (like slanted quotes).
			$search = array(chr(145), chr(146), chr(147), chr(148), chr(151), chr(133));      
			$replace = array("'", "'", '"', '"', '-', "...");
			
			$dataToFilter = str_replace($search, $replace, $dataToFilter);
		}
		else if($filterType == FILTER_SANITIZE_URL){
			$dataToFilter = filter_var($dataToFilter, FILTER_SANITIZE_URL);
		}
		else if($filterType == FILTER_SANITIZE_WORD_OR_NUMBER_NO_SPACES){
			
			$matches = array();
			
			if(preg_match("/([a-z0-9_]+)/i", $dataToFilter, $matches)){
				$dataToFilter = $matches[1];
			}
			else{
				$dataToFilter = "";
			}
		}
		else{
			throw new Exception("Error in method GetInput. The Filter Type has not been defined yet: " . $filterType);
		}
		
		return $dataToFilter;
	}

	// Gets specified variable out of the PHP session managment.
	// Returns nullValue if variable not present, otherwise it returns processed value.
	static function getSessionVar( $vname, $nullValue = null ){
		
		if(!isset($_SESSION)){
			return $nullValue;
		}
		
		if( !array_key_exists( $vname, $_SESSION )){
			return $nullValue;
		}

		return $_SESSION[$vname];
	}

	static function setSessionVar( $vname, $value){

		// Do not throw an Exception... because this could cause an Exception within the Exception Handler.
		if(!isset($_SESSION)){
			exit("Can not set a session variable if the session has not started.");
		}
		
		$_SESSION[$vname] = $value;
	}
	
	// Similar to GetSessionVar... but gets server variable.
	// Not all server variables are populated depending on apache, etc.
	// For example, sometimes the HTTP Referrer is NULL
	static function getServerVar( $vname, $nullValue = null ){
		
		if(!isset($_SERVER)){
			return $nullValue;
		}
		
		if( !array_key_exists( $vname, $_SERVER )){
			return $nullValue;
		}

		return trim($_SERVER[$vname]);
	}

	static function getCookie( $cname, $nullValue = null ){

		if( !array_key_exists( $cname, $_COOKIE )){
			return $nullValue;
		}

		return trim( $_COOKIE[$cname]);
	}
	
	static function setCookie($cname, $cvalue, $numDays = 150){
		
		$cookieTime = time()+60*60*24*$numDays;
		setcookie ($cname, $cvalue, $cookieTime);
	}

	static function breakOutOfSecureMode(){
		
		if ( isset($_SERVER['HTTPS']) ) {
			
			if(strtolower($_SERVER['HTTPS']) == 'on'){
				session_write_close();
				header ('Location: http://'.$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'], true, 302);
				exit;
			}
		}
	}

	static function checkIfInSecureMode(){

		if ( !isset($_SERVER['HTTPS']) || strtolower($_SERVER['HTTPS']) != 'on' ) {
		  return false;
		}
		else{
			return true;
		}
	}

	// This good method to prevent Cross-Site-Forgery-Request attacks.
	// Set's a unique security code in the users session and returns that code.  
	// You should put that code into a hidden input in an HTML format.
	// Call check checkFormSecurityCode() after the form is sumbitted, before saving to the Database.
	private static $formSecurityCode;
	static function getFormSecurityCode(){
		
		$securitySalt = "blah,blah,blah";

		// Only generate 1 unique form security code per script execution. 
		// Otherwise if it is called for a whole bunch of links in a loop it could set tons of session variables.  
		if(empty(self::$formSecurityCode))
			self::$formSecurityCode = md5(time() . uniqid() . $securitySalt);
		else
			return self::$formSecurityCode;
		
		// Add to an array stored in the Session.
		$codesArr = WebUtil::GetSessionVar("FormSecurityCodes", array());

		$codesArr[] = self::$formSecurityCode;
	
		WebUtil::SetSessionVar("FormSecurityCodes", $codesArr);

		return self::$formSecurityCode;
	}
	
	// Looks in the user's $_REQUEST vars looking for variable called "form_sc"
	// If that variable is not found, an erorr message will be written.  
	// Then it makes sure that the user has a session variable set with a matching code or it will fail.
	static function checkFormSecurityCode(){

		$securityCodeInForm = WebUtil::GetInput("form_sc", FILTER_SANITIZE_WORD_OR_NUMBER_NO_SPACES);
		
		if(empty($securityCodeInForm)){
			throw new Exception("Error in method checkFormSecurityCode. It was not found within the Request.");
		}
		
		$codesArr = WebUtil::GetSessionVar("FormSecurityCodes", array());
		
		if(!in_array($securityCodeInForm, $codesArr)){
			throw new Exception("The action you tried to perform could not be completed. Your browser window may have been sitting idle for too long.");
		}
		
		// Don't let a users session file grow out of control.  
		// At the same time we want to give users the ability to do stuff in multiple windows and use Security Codes from within Iframes.
		if(sizeof($codesArr) > 70){
			
			$numToShift = sizeof($codesArr) - 70;
			
			for($i=0; $i<$numToShift; $i++){
				array_shift($codesArr);
			}
				
			WebUtil::SetSessionVar("FormSecurityCodes", $codesArr);
		}
	}
	
	static function htmlOutput($str){
		
		// Read more at... http://ha.ckers.org/blog/20070327/htmlspecialchars-strikes-again/
		return htmlentities($str, ENT_QUOTES, 'ISO-8859-1');
	}
	
	static function print404Error(){
		header("HTTP/1.0 404 Not Found");
		
		// There are some '\t'abs in the formatting to make it look neat in the codebase that need to be removed before outputting.
		exit(preg_replace('/\t/', '', '
		<html><head>
		<title>404 Not Found</title>
		</head><body>
		<h1>404 Not Found</h1>
		<p>The URL was not found on this server.</p>
		</body>
		</html>'));

	}

	static function runInSecureModeHTTPS(){
		if(!Constants::getDevelopmentServer()){
			if ( !isset($_SERVER['HTTPS']) || strtolower($_SERVER['HTTPS']) != 'on' ) {
			   header ('Location: https://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI']);
			   exit;
			}
		}
	}
}

	
	
